<?php

// Swiff Chart Generator 2 : PHP Module for Unix

// Copyright (C) 2002, GlobFX Technologies, SARL.
// All Rights Reserved.
// This is UNPUBLISHED PROPRIETARY SOURCE CODE of GlobFX Technologies, SARL.
// The contents of this file may not be disclosed to third parties, copied or
// duplicated in any form, in whole or in part, without the prior written
// permission of GlobFX Technologies, SARL.

class SwiffChart
{
  var $width;
  var $height;
  var $frame_rate;
  var $animate;
  var $looping;
  var $swf_compressed;
  var $swf_protected;

  var $separators;
  var $ignore_multiple_sep;
  var $decimal_sep;
  var $thousands_sep;

  var $install_dir;
  var $style;

  var $title;
  var $subtitle;

  var $categs;
  var $series_captions;
  var $series_x_values;
  var $series_y_values;

  var $axis_min_value;
  var $axis_max_value;
  var $axis_cross_value;
  var $axis_title;

  var $cache_name;
  var $private_cache_dir;
  var $max_cache_size;
  var $fonts_dir_list;

  var $use_cache;

  var $output_format;
  var $jpg_quality;
  var $png_comp_level;

  var $document_root;
  var $no_cache;

  function SwiffChart()
    {
    $this->ClearAll();

    $this->install_dir= "/usr/local/SwiffChart";

    $this->cache_name= "";
    $this->private_cache_dir= "";
    $this->max_cache_size= -1;
    $this->use_cache= false;

    $this->output_format= 0;
    $this->jpg_quality= 75;
    $this->png_comp_level= 3;

    global $HTTP_SERVER_VARS;
    if( isset($HTTP_SERVER_VARS) && isset($HTTP_SERVER_VARS['DOCUMENT_ROOT']) )
      $this->document_root= $HTTP_SERVER_VARS['DOCUMENT_ROOT'];
     else
      {
      global $DOCUMENT_ROOT;
      $this->document_root= $DOCUMENT_ROOT;
      }

    $this->no_cache= true;
    }

  function Release()
    {}

  function ClearAll()
    {
    $this->width= 0;
    $this->height= 0;
    $this->frame_rate= 0;
    $this->animate= true;
    $this->looping= -1;
    $this->has_looping= false;
    $this->swf_compressed= false;
    $this->swf_protected= true;

    $this->separators= ';';
    $this->ignore_multiple_sep= false;
    $this->decimal_sep= '';
    $this->thousands_sep= '';

    $this->style= "";

    $this->title= "<unset>";
    $this->subtitle= "<unset>";

    $this->categs= array();
    $this->series_captions= array();
    $this->series_x_values= array();
    $this->series_y_values= array();

    $this->axis_min_value= array();
    $this->axis_max_value= array();
    $this->axis_cross_value= array();
    $this->axis_title= array();
    }

  // General Parameters

  function SetWidth( $_width )
    {
    $this->width= $_width;
    }

  function SetHeight( $_height )
    {
    $this->height= $_height;
    }

  function GetWidth()
    {
    return $this->width;
    }

  function GetHeight()
    {
    return $this->height;
    }

  function SetFrameRate( $_frame_rate )
    {
    $this->frame_rate= $_frame_rate;
    }

  function GetFrameRate()
    {
    return $this->frame_rate;
    }

  function AnimateChart( $_animate )
    {
    $this->animate= (bool) $_animate;
    }

  function IsAnimated()
    {
    return $this->animate;
    }

  function SetLooping( $_looping )
    {
    $this->looping= ((bool) $_looping) ? 1 : 0;
    }

  function GetLooping()
    {
    return (bool) $this->looping;
    }

  function CompressSWF( $_swf_compressed )
    {
    $this->swf_compressed= (bool) $_swf_compressed;
    }

  function IsSWFCompressed()
    {
    return $this->swf_compressed;
    }

  function ProtectSWF( $_swf_protected )
    {
    $this->swf_protected= (bool) $_swf_protected;
    }

  function IsSWFProtected()
    {
    return $this->swf_protected;
    }

  // Separators

  function SetSeparators( $_separators, $_ignore_multiple_sep )
    {
    if( $_separators == '' )
      $_separators= ';';
    $this->separators= $_separators;
    $this->ignore_multiple_sep= (bool) $_ignore_multiple_sep;
    }
  function GetSeparators()
    {
    return $this->separators;
    }

  function SetLocaleInfo( $type, $data )
    {
    switch($type)
      {
      case 1: $this->decimal_sep=$data; break;
      case 2: $this->thousands_sep=$data; break;
      default:
        die("SetLocaleInfo(): invalid type ($type)");
      }
    }

  // Titles

  function SetTitle( $_title )
    {
    $this->title= $_title;
    }

  function GetTitle()
    {
    return $this->title;
    }

  function SetSubtitle( $_subtitle )
    {
    $this->subtitle= $_subtitle;
    }

  function GetSubtitle()
    {
    return $this->subtitle;
    }

  // Categories

  function SetCategoriesFromArray( $categories )
    {
    $this->categs= array();

    if( empty($categories) )
      return;

    ksort($categories);
    end($categories);
    $max_index= key($categories);

    for( $i= 0; $i <= $max_index; ++$i )
      $this->categs[$i]= $categories[$i];
    }

  function SetCategoriesFromString( $categories )
    {
    if( $this->ignore_multiple_sep )
      $categories= preg_replace( '/([\''.$this->separators.'\']+)/', $this->separators[0], $categories );

    $categ_array= split( '['.$this->separators.']', $categories );
    $this->SetCategoriesFromArray($categ_array);
    }

  function GetCategory( $index )
    {
    if( $index < 0 || $index >= count($this->categs) )
      die("GetCategory(): index $index is out of bounds [0,".count($this->categs)."]");

    return $this->categs[$index];
    }

  // Series Captions

  function SetSeriesCaptionsFromArray($captions)
    {
    $this->series_captions= array();
    if( empty($captions) )
      return;

    ksort($captions);
    end($captions);
    $max_index= key($captions);
    reset($captions);
    $min_index= key($captions);

    for( $i= 0; $i <= $max_index; ++$i )
      $this->series_captions[$i]= $captions[$min_index+$i];

    $n= $i;
    while( $this->GetSeriesCount() < $n )
      $this->AddSeries();
    }

  function SetSeriesCaptionsFromString($captions)
    {
    if( $this->ignore_multiple_sep )
      $captions= preg_replace( '/([\''.$this->separators.'\']+)/', $this->separators[0], $captions );

    $categ_array= split( '['.$this->separators.']', $captions );
    $this->SetSeriesCaptionsFromArray($categ_array);
    }

  function SetSeriesCaption( $index, $caption )
    {
    if( $index < 0 || $index >= $this->GetSeriesCount() )
      die("SetSeriesCaption(): index $index is out of bounds [0,".($this->GetSeriesCount()-1)."]");

    $this->series_captions[$index]= $caption;
    }

  function GetSeriesCaption( $index )
    {
    if( $index < 0 || $index >= $this->GetSeriesCount() )
      die("GetSeriesCaption(): index $index is out of bounds [0,".($this->GetSeriesCount()-1)."]");

    return $this->series_captions[$index];
    }

  // Series Values

  function AddSeries()
    {
    $n= $this->GetSeriesCount();
    $this->series_y_values[$n]= array();
    $this->series_x_values[$n]= array();
    }

  function GetValuesCount()
    {
    $max_n= count($this->categs);
    for( $i= 0; $i < count($this->series_y_values); ++$i )
      {
      $n= count($this->series_y_values[$i]);
      if( $n > $max_n )
        $max_n= $n;

      $n= count($this->series_x_values[$i]);
      if( $n > $max_n )
        $max_n= $n;
      }

    return $max_n;
    }

  function GetSeriesCount()
    {
    return count($this->series_y_values);
    }

  function GetSeriesValue( $index, $valueindex )
    {
    return $this->GetSeriesYValue($index,$valueindex);
    }

  function GetSeriesXValue( $index, $valueindex )
    {
    if( $index < 0 || $index >= $this->GetSeriesCount() )
      die("GetSeriesXValue(): index $index is out of bounds [0,".($this->GetSeriesCount()-1)."]");

    $s= $this->series_x_values[$index];
    if( $index < 0 || $index >= count($s) )
      die("GetSeriesXValue(): value index $valueindex is out of bounds [0,".(count($s)-1)."]");

    return $s[$valueindex];
    }

  function GetSeriesYValue( $index, $valueindex )
    {
    if( $index < 0 || $index >= $this->GetSeriesCount() )
      die("GetSeriesYValue(): index $index is out of bounds [0,".($this->GetSeriesCount()-1)."]");

    $s= $this->series_y_values[$index];
    if( $index < 0 || $index >= count($s) )
      die("GetSeriesYValue(): value index $valueindex is out of bounds [0,".(count($s)-1)."]");

    return $s[$valueindex];
    }

  function SetSeriesValuesFromArray( $index, $values )
    {
    return $this->SetSeriesYValuesFromArray( $index, $values );
    }

  function SetSeriesValuesFromString( $index, $values )
    {
    return $this->SetSeriesYValuesFromString( $index, $values );
    }

  function SetSeriesXValuesFromArray($index, $values)
    {
    if( $index < 0 || $index >= $this->GetSeriesCount() )
      die("SetSeriesXValuesFromArray(): index $index is out of bounds [0,".($this->GetSeriesCount()-1)."]");

    $this->series_x_values[$index]= array();

    if( empty($values) )
      return;

    ksort($values);
    end($values);
    $max_index= key($values);

    for( $i= 0; $i <= $max_index; ++$i )
      $this->series_x_values[$index][$i]= $values[$i];
    }

  function SetSeriesXValuesFromString($index, $values)
    {
    if( $this->ignore_multiple_sep )
      $values= preg_replace( '/([\''.$this->separators.'\']+)/', $this->separators[0], $values );

    $values_array= split( '['.$this->separators.']', $values );

    $this->SetSeriesXValuesFromArray($index,$values_array);
    }

  function SetSeriesYValuesFromArray( $index, $values )
    {
    if( $index < 0 || $index >= $this->GetSeriesCount() )
      die("SetSeriesYValuesFromArray(): index $index is out of bounds [0,".($this->GetSeriesCount()-1)."]");

    $this->series_y_values[$index]= array();

    if( empty($values) )
      return;

    ksort($values);
    end($values);
    $max_index= key($values);

    for( $i= 0; $i <= $max_index; ++$i )
      $this->series_y_values[$index][$i]= $values[$i];
    }

  function SetSeriesYValuesFromString( $index, $values )
    {
    if( $this->ignore_multiple_sep )
      $values= preg_replace( '/([\''.$this->separators.'\']+)/', $this->separators[0], $values );

    $values_array= split( '['.$this->separators.']', $values );

    $this->SetSeriesYValuesFromArray($index,$values_array);
    }

  function SetSeriesXYValuesFromString( $index, $values )
    {
    if( $index < 0 || $index >= $this->GetSeriesCount() )
      die("SetSeriesXYValuesFromString(): index $index is out of bounds [0,".($this->GetSeriesCount()-1)."]");

    $this->series_x_values[$index]= array();
    $this->series_y_values[$index]= array();

    if( $this->ignore_multiple_sep )
      $values= preg_replace( '/([\''.$this->separators.'\']+)/', $this->separators[0], $values );
    $values_array= split( '['.$this->separators.']', $values );

    if( empty($values_array) )
      return;

    ksort($values_array);
    end($values_array);
    $max_index= key($values_array) / 2;

    for( $i= 0; $i <= $max_index; ++$i )
      {
      $this->series_x_values[$index][$i]= $values_array[2*$i];
      $this->series_y_values[$index][$i]= $values_array[2*$i+1];
      }
    }

  // Axes Parameters

  function SetAxisMinValue( $axis_number, $min_value )
    {
    if( $axis_number < 0 || $axis_number > 1 )
      die("SetAxisMinValue(): axis number axis_number is out of bounds [0,1]");
    $this->axis_min_value[$axis_number]= $min_value;
    }

  function SetAxisMaxValue( $axis_number, $max_value )
    {
    if( $axis_number < 0 || $axis_number > 1 )
      die("SetAxisMaxValue(): axis number axis_number is out of bounds [0,1]");
    $this->axis_max_value[$axis_number]= $max_value;
    }

  function SetAxisCrossValue( $axis_number, $cross_value )
    {
    if( $axis_number < 0 || $axis_number > 1 )
      die("SetAxisCrossValue(): axis number axis_number is out of bounds [0,1]");
    $this->axis_cross_value[$axis_number]= $cross_value;
    }

  function ResetAxisBounds( $axis_number )
    {
    $this->axis_min_value= array();
    $this->axis_max_value= array();
    $this->axis_cross_value= array();
    }

  function SetAxisTitle( $axis_number, $title )
    {
    if( $axis_number < 0 || $axis_number > 1 )
      die("SetAxisTitle(): axis number axis_number is out of bounds [0,1]");
    $this->axis_title[$axis_number]= $title;
    }

  function GetAxisTitle( $axis_number )
    {
    if( $axis_number < 0 || $axis_number > 1 )
      die("GetAxisTitle(): axis number axis_number is out of bounds [0,1]");
    return $this->axis_title[$axis_number];
    }

  // Import Parameters and Values from Query

  function SetDataFromQuery()
    {
    if( isset($_SERVER) && isset($_SERVER['QUERY_STRING']) )
      $qs= $_SERVER['QUERY_STRING'];
     else
      {
      global $QUERY_STRING;
      $qs= $QUERY_STRING;
      }

    $n_series=0;
    $qa= split("&",$qs);
    foreach($qa as $vv)
      {
      $vv2= split("=",$vv);
      if( count($vv2) >= 2 )
        list($var,$val)= $vv2;
       else
        {
        $var= $vv2[0];
        $val= "";
        }

      $var= urldecode($var);
      $val= urldecode($val);

      if( strcasecmp($var,"animate") == 0 )
        {
        $animate= ($val == "0" ||
                   strcasecmp($val,"no") == 0 ||
                   strcasecmp($val,"false") == 0) ? false : true;
        $this->AnimateChart($animate);
        continue;
        }

      if( strcasecmp($var,"width") == 0 )
        {
        $this->SetWidth($val);
        continue;
        }

      if( strcasecmp($var,"height") == 0 )
        {
        $this->SetHeight($val);
        continue;
        }

      if( strcasecmp($var,"fps") == 0 )
        {
        $this->SetFrameRate($val);
        continue;
        }

      if( strcasecmp($var,"title") == 0 )
        {
        $this->SetTitle($val);
        continue;
        }

      if( strcasecmp($var,"categories") == 0 )
        {
        $this->SetCategoriesFromString($val);
        continue;
        }

      if( strcasecmp($var,"captions") == 0 )
        {
        $this->SetSeriesCaptionsFromString($val);
        continue;
        }

      if( preg_match("/^seriesxy/i",$var) )
        {
        $sva= split( '['.$this->separators.']', $val);
        $series_values= "";
        for( $i= 0; $i < count($sva); ++$i )
          $series_values .= (($i==0) ? "" : $this->separators[0]) . $sva[$i];

        $this->AddSeries();
        $this->SetSeriesXYValuesFromString($n_series,$series_values);
        $n_series++;
        continue;
        }

      if( preg_match("/^series/i",$var) )
        {
        $sva= split( '['.$this->separators.']', $val);
        $series_values= "";
        for( $i= 0; $i < count($sva); ++$i )
          $series_values .= (($i==0) ? "" : $this->separators[0]) . $sva[$i];

        $this->AddSeries();
        $this->SetSeriesValuesFromString($n_series,$series_values);
        $n_series++;
        continue;
        }
      }
    }

  // Import Values from Text File

  function SetDataFromTxtFile( $filename, $seriesincolumn, $titlesinfirstrow, $titlesinfirstcolumn )
    {
    $fp= fopen($filename,"rt");
    if( ! $fp )
      die("SetDataFromTxtFile(): Unable to open file $filename");

    $this->series_x_values= array();
    $this->series_y_values= array();

    $lines= array();
    $n_lines= 0;
    while( ! feof($fp) )
      {
      $ln= chop(fgets($fp,4096));

      if( $this->ignore_multiple_sep )
        $ln= preg_replace( '/([\''.$this->separators.'\']+)/', $this->separators[0], $ln );

      $lna= strlen($ln) ? split( '['.$this->separators.']', $ln) : array();

      $lines[$n_lines]= $lna;
      ++$n_lines;
      }

    fclose($fp);

    for( $i= count($lines)-1; $i >= 0; --$i )
      {
      $row_is_empty= true;
      foreach( $lines[$i] as $key => $value )
        {
        if( trim($value) != '' )
          {
          $row_is_empty= false;
          break;
          }
        }

      if( $row_is_empty )
        array_pop($lines);
       else
        break;
      }

    if( $titlesinfirstrow )
      {
      if( $titlesinfirstcolumn )
        $this->SetTitle(array_shift($lines[0]));
      if( $seriesincolumn )
        $this->SetSeriesCaptionsFromArray($lines[0]);
       else
        $this->SetCategoriesFromArray($lines[0]);
      array_shift($lines);
      }

    if( $titlesinfirstcolumn )
      {
      $c= array();
      for( $i= 0; $i < count($lines); ++$i )
        $c[$i]= array_shift($lines[$i]);

      if( $seriesincolumn )
        $this->SetCategoriesFromArray($c);
       else
        $this->SetSeriesCaptionsFromArray($c);
      }

    if( $seriesincolumn )
      {
      $n_series= 0;
      for( $i= 0; $i < count($lines); ++$i )
        {
        if( count($lines[$i]) > $n_series )
          $n_series= count($lines[$i]);
        }
      while( $this->GetSeriesCount() < $n_series )
        $this->AddSeries();

      for( $j= 0; $j < $n_series; ++$j )
        {
        $c= array();
        for( $i= 0; $i < count($lines); ++$i )
          $c[$i]= $lines[$i][$j];
        $this->setSeriesValuesFromArray($j,$c);
        }
      }
     else
      {
      while( $this->GetSeriesCount() < count($lines) )
        $this->AddSeries();
      for( $i= 0; $i < count($lines); ++$i )
        $this->setSeriesValuesFromArray($i,$lines[$i]);
      }
    }

  function SetXYDataFromTxtFile( $filename, $seriesincolumn, $titlesinfirstrow, $titlesinfirstcolumn )
    {
    $fp= fopen($filename,"rt");
    if( ! $fp )
      die("SetDataFromTxtFile(): Unable to open file $filename");

    $this->series_x_values= array();
    $this->series_y_values= array();

    $lines= array();
    $n_lines= 0;
    while( ! feof($fp) )
      {
      $ln= chop(fgets($fp,4096));

      if( $this->ignore_multiple_sep )
        $ln= preg_replace( '/([\''.$this->separators.'\']+)/', $this->separators[0], $ln );

      $lna= strlen($ln) ? split( '['.$this->separators.']', $ln) : array();

      $lines[$n_lines]= $lna;
      ++$n_lines;
      }

    fclose($fp);

    for( $i= count($lines)-1; $i >= 0; --$i )
      {
      $row_is_empty= true;
      foreach( $lines[$i] as $key => $value )
        {
        if( trim($value) != '' )
          {
          $row_is_empty= false;
          break;
          }
        }

      if( $row_is_empty )
        array_pop($lines);
       else
        break;
      }

    if( $titlesinfirstrow )
      {
      if( $titlesinfirstcolumn )
        $this->SetTitle(array_shift($lines[0]));
      if( $seriesincolumn )
        {
        $a= array();
        $n= (int) ((count($lines[0])+1) / 2);
        for( $i= 0; $i < $n; ++$i )
          $a[$i]= $lines[0][2*$i];
        $this->SetSeriesCaptionsFromArray($a);
        }
       else
        $this->SetCategoriesFromArray($lines[0]);
      array_shift($lines);
      }

    if( $titlesinfirstcolumn )
      {
      $c= array();
      for( $i= 0; $i < count($lines); ++$i )
        $c[$i]= array_shift($lines[$i]);

      if( $seriesincolumn )
        $this->SetCategoriesFromArray($c);
       else
        {
        $a= array();
        $n= (int) ((count($c)+1) / 2);
        for( $i= 0; $i < $n; ++$i )
          $a[$i]= $c[2*$i];
        $this->SetSeriesCaptionsFromArray($a);

        }
      }

    if( $seriesincolumn )
      {
      $n_series= 0;
      for( $i= 0; $i < count($lines); ++$i )
        {
        if( count($lines[$i]) > $n_series )
          $n_series= count($lines[$i]);
        }
      while( $this->GetSeriesCount() < $n_series )
        $this->AddSeries();

      for( $j= 0; $j < $n_series; ++$j )
        {
        $c= array();
        for( $i= 0; $i < count($lines); ++$i )
          $c[$i]= $lines[$i][$j];
        $this->setSeriesValuesFromArray($j,$c);
        }
      }
     else
      {
      while( $this->GetSeriesCount() < count($lines) )
        $this->AddSeries();
      for( $i= 0; $i < count($lines); ++$i )
        $this->setSeriesValuesFromArray($i,$lines[$i]);
      }
    }


  // External Chart Generation

  function mangle( $str )
    {
    return str_replace( '-', '%2D', rawurlencode($str) );
    }

  function prepare_cmd_line( $html_tag = false, $output_location = false )
    {
    $exe= "$this->install_dir/swfchart";

    is_executable($exe) || die("Cannot execute $exe");

    $cmd= $exe;

    if( $this->style != "" )
      $cmd .= " -s '".$this->mangle($this->style)."'";

    if( $this->width >= 1 )
      $cmd .= " --width '".$this->mangle($this->width)."'";
    if( $this->height >= 1 )
      $cmd .= " --height '".$this->mangle($this->height)."'";
    if( $this->frame_rate >= 1 )
      $cmd .= " --fps '".$this->mangle($this->frame_rate)."'";

    if( !$this->animate )
      $cmd .= " -a false";
    if( !$this->swf_protected )
      $cmd .= " --protect false";
    if( $this->swf_compressed )
      $cmd .= " --compress true";

    if( $this->looping >= 0)
      $cmd .= " --looping ".($this->looping ? "true" : "false");

    $cmd .= " -tr false -tc false -sr";

    if( $this->title != "<unset>" )
      $cmd .= " --title '".$this->mangle($this->title)."'";
    if( $this->subtitle != "<unset>" )
      $cmd .= " --subtitle '".$this->mangle($this->subtitle)."'";

    if( $this->decimal_sep != '' )
      $cmd .= " -ds '".$this->mangle($this->decimal_sep)."'";
    if( $this->thousands_sep != '' )
      $cmd .= " -ts '".$this->mangle($this->thousands_sep)."'";

    $n= count($this->categs);
    if( $n > 0 )
      {
      $c= '';
      for( $i= 0; $i < $n; ++$i )
        $c .= (($i > 0) ? $this->separators[0] : '') . $this->categs[$i];
      $cmd .= " -c '".$this->mangle($c)."'";
      }

    $n= count($this->series_captions);
    if( $n > 0 )
      {
      $c= '';
      for( $i= 0; $i < $n; ++$i )
        $c .= (($i > 0) ? $this->separators[0] : '') . $this->series_captions[$i];
      $cmd .= " -i '".$this->mangle($c)."'";
      }

    $n= count($this->series_y_values);
    if( $n > 0 )
      {
      $c= '';

      $has_x_values= false;
      for( $i= 0; $i < $n; ++$i )
        {
        if( count($this->series_x_values[$i]) > 0 )
          {
          $has_x_values= true;
          break;
          }
        }

      for( $i= 0; $i < $n; ++$i )
        {
        if( $has_x_values )
          {
          $xs= $this->series_x_values[$i];
          for( $j= 0; $j < count($xs); ++$j )
            $c .= (($j > 0) ? $this->separators[0] : '') . $xs[$j];
          $c .= '\\n';
          }

        $ys= $this->series_y_values[$i];
        for( $j= 0; $j < count($ys); ++$j )
          $c .= (($j > 0) ? $this->separators[0] : '') . $ys[$j];

        if( $i+1 < $n )
          $c .= '\\n';
        }

      $cmd .= " -n '".$this->mangle($c)."'";
      }

    if( $this->axis_title[0] != '' )
      $cmd .= " --haxis-title '".$this->mangle($this->axis_title[0])."'";
    if( $this->axis_title[1] != '' )
      $cmd .= " --vaxis-title '".$this->mangle($this->axis_title[1])."'";

    if( $this->axis_min_value[0] != '' )
      $cmd .= " --axis-min 0 '".$this->mangle($this->axis_min_value[0])."'";
    if( $this->axis_min_value[1] != '' )
      $cmd .= " --axis-min 1 '".$this->mangle($this->axis_min_value[1])."'";

    if( $this->axis_max_value[0] != '' )
      $cmd .= " --axis-max 0 '".$this->mangle($this->axis_max_value[0])."'";
    if( $this->axis_max_value[1] != '' )
      $cmd .= " --axis-max 1 '".$this->mangle($this->axis_max_value[1])."'";

    if( $this->axis_cross_value[0] != '' )
      $cmd .= " --axis-cross 0 '".$this->mangle($this->axis_cross_value[0])."'";
    if( $this->axis_cross_value[1] != '' )
      $cmd .= " --axis-cross 1 '".$this->mangle($this->axis_cross_value[1])."'";

    if( $this->separators[0] != ';' )
      $cmd .= " -d '".$this->mangle($this->separators[0])."'";

    if( $html_tag || $output_location || $this->use_cache )
      {
      if( $html_tag )
        $cmd .= ' --html-tag';
       else
        {
        if( $output_location )
          $cmd .= ' --output-loc';
         else
          $cmd .= ' --use-cache';
        }

      $cmd .= " --doc-root '".$this->mangle($this->document_root)."'";

      if( $this->cache_name != "" )
        $cmd .= " --cache-name '".$this->mangle($this->cache_name)."'";

      if( $this->private_cache_dir != "" )
        $cmd .= " --private-cache-dir '".$this->mangle($this->private_cache_dir)."'";

      if( $this->max_cache_size >= 0 )
        $cmd .= " --max-cache-size '".$this->max_cache_size."'";
      }

    if( $this->fonts_dir_list != "" )
      $cmd .= " --fonts '".$this->mangle($this->fonts_dir_list)."'";

    if( $this->output_format == 1 )
      {
      $cmd .= ' --format jpg';
      if( $this->jpg_quality != 75 )
        $cmd .= " --jpg-quality '".$this->jpg_quality."'";
      }
    else
    if( $this->output_format == 2 )
      {
      $cmd .= ' --format png';
      if( $this->png_comp_level != 3 )
        $cmd .= " --png-comp-level '".$this->png_comp_level."'";
      }

    $cmd .= " --install-dir '".$this->install_dir."'";

    return $cmd;
    }

  function SetCacheName( $_cache_name )
    {
    $this->cache_name= $_cache_name;
    }
  function SetPrivateCacheDir( $private_cache_dir )
    {
    $this->private_cache_dir= $private_cache_dir;
    }
  function SetMaxCacheSize( $_max_cache_size )
    {
    $this->max_cache_size= $_max_cache_size;
    }

  function ClearCache()
    {
    $cmd= $this->prepare_cmd_line(true);
    $cmd .= ' --clear-cache';

    shell_exec($cmd);
    }

  function UseCache( $_use_cache )
    {
    $this->use_cache= $_use_cache;
    }

  function SetTTFontsDir( $dir )
    {
    $this->fonts_dir_list= $dir;
    }

  function SetOutputFormat( $format )
    {
    switch( strtolower($format) )
      {
      case 'swf': $this->output_format= 0; break;
      case 'jpg': $this->output_format= 1; break;
      case 'png': $this->output_format= 2; break;
      default:
        die("SetOutputFormat(): Invalid output format \"$format\"");
        return;
      }
    }

  function GetHTTPContentType()
    {
    switch($this->output_format)
      {
      case 0: return 'application/x-shockwave-flash';
      case 1: return 'image/jpeg';
      case 2: return 'image/png';
      }
    return '';
    }

  function ExportAsResponse()
    {
    $cmd= $this->prepare_cmd_line();

    $swf= shell_exec($cmd);
    if( ! $swf )
      die("ExportAsResponse(): An error occurred when running: ".rawurldecode($cmd));

    Header( 'Content-Type: '.$this->GetHTTPContentType() );
    if( $this->no_cache )
      {
      Header("Expires: Mon, 01 Jan 1990 00:00:00 GMT");
      Header("Pragma: no-cache");
      }
    echo $swf;
    }

  function ExportAsBinary()
    {
    $cmd= $this->prepare_cmd_line();
    return shell_exec($cmd);
    }

  function ExportAsBase64()
    {
    return base64_encode($this->ExportAsBinary());
    }

  function ExportAsFile( $filename )
    {
    $cmd= $this->prepare_cmd_line();
    $cmd .= " -o ".$this->mangle($filename);

    $fp= fopen($filename,"wb");
    if( ! $fp )
      die("ExportAsFile(): Cannot write to file $filename");
    fclose($fp);
    unlink($filename);

    shell_exec($cmd);
    }

  function SetJPGQuality($quality)
    {
    $this->jpg_quality= $quality;
    }

  function SetPNGCompLevel($comp_level)
    {
    $this->png_comp_level= $comp_level;
    }

  function GetHTMLTag()
    {
    $cmd= $this->prepare_cmd_line( true/*html_tag*/ );
    return shell_exec($cmd);
    }

  function GetOutputLocation()
    {
    $cmd= $this->prepare_cmd_line( false/*html_tag*/, true/*output_location*/ );
    return shell_exec($cmd);
    }

  // Miscellaneous

  function LoadStyle( $_style_filename )
    {
    if( substr_count($_style_filename,"/") == 0 || $_style_filename[0] != '/' )
      $_style_filename= $this->install_dir."/styles/".$_style_filename;

    if( substr_count(basename($_style_filename),".") == 0 )
      $_style_filename .= '.scs';

    (file_exists($_style_filename) && is_readable($_style_filename))
          || die("Unable to open file $_style_filename");

    $this->style= $_style_filename;
    }

  function GetVersion()
    {
    $exe= "$this->install_dir/swfchart";
    is_executable($exe) || die("Cannot execute $exe");
    return shell_exec( "$exe -rawversion" );
    }

  function SetInstallDir( $_install_dir )
    {
    $this->install_dir= $_install_dir;
    }

  function SetDocumentRoot( $_document_root )
    {
    $this->document_root= $_document_root;
    }
}
?>
